// $Id: gucOverlay.js,v 1.3 2006/03/09 21:04:14 gramakri Exp $

/* Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Girish Ramakrishnan.
 * Portions created by the Initial Developer are Copyright (C) 2004
 * by the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): Girish Ramakrishnan <gramakri@uiuc.edu>
 *                 Tahera <taheraab@gmail.com>
 * Thanks to Christoph Berg for his one liner that made all the difference
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 */

var gInThunderbird = false;
var gInChatzilla = false;
var gInFirefox = false;

var gucPrefs = Components.classes["@mozilla.org/preferences-service;1"].
             getService(Components.interfaces.nsIPrefService).getBranch("guc.");

var gucStringBundle;

var observerService = Components.classes["@mozilla.org/observer-service;1"].
             getService(Components.interfaces.nsIObserverService);

// register callback everytime we are created
window.addEventListener("load", gucInit, false);

var observer = {
    // This function is called when the extension settings are changed by the user
    observe: function(subject, topic, data) {
        gucUpdateContextMenu();
    }
};

function gucInit()
{
    if (navigator.userAgent.search(/Thunderbird/gi) != -1) {
        gInThunderbird = true;
        var popup = document.getElementById("messagePaneContext");
        if (popup)
            popup.addEventListener("popupshowing", onGucPopup, false);
        popup = document.getElementById("msgComposeContext");
        if (popup)
            popup.addEventListener("popupshowing", onGucPopup, false);
    } else if (document.getElementById("chatzilla-window") != null) {
        gInChatzilla = true;
        var popup = document.getElementById("context:messages");
        if (popup != null)
            popup.addEventListener("popupshowing", onGucPopup, false);
    } else {
        gInFirefox = true;
        var popup = document.getElementById("contentAreaContextMenu");
        if (popup)
            popup.addEventListener("popupshowing", onGucPopup, false);
    } 
        
    gucStringBundle = document.getElementById("guc_string_bundle");

    if (!gucPrefs.prefHasUserValue("groupRules")) {
        // This preference was added in 1.0. Defaulted to true to match pre1.0
        gucPrefs.setBoolPref("groupRules", true);
    }

    if (!gucPrefs.prefHasUserValue("contextMenuText")) {
        gucPrefs.setCharPref("contextMenuText", 
                gucStringBundle.getString("guc.default.contextMenuText"));
    }

    if (!gucPrefs.prefHasUserValue("menutext1")) {
        var i = 1;
        var mt;
        while (true) {
           try { mt = gucStringBundle.getString("guc.default.menutext" + i);  }
           catch (ex) { break; }
           if (mt == "") break;
           gucPrefs.setCharPref("menutext"+i, mt);
           var pat = gucStringBundle.getString("guc.default.searchpattern" + i);
           gucPrefs.setCharPref("searchpattern"+i, pat);
           var url = gucStringBundle.getString("guc.default.url" + i);
           gucPrefs.setCharPref("url"+i, url);
           i = i + 1;
        }
    }

    observerService.addObserver(observer, "guc-settings", false);
    
    gucUpdateContextMenu();
}

function gucUpdateContextMenu()
{
       //Create guc context menu items depending upon whether it's the browser windows or mail windows
    if (gInThunderbird) {
        var popup = document.getElementById("messagePaneContext");
        if (popup != null)  gucCreateContextMenu(popup);
        var popup = document.getElementById("msgComposeContext");
        if (popup != null)  gucCreateContextMenu(popup);
    } else if (gInChatzilla) {
         var popup = document.getElementById("context:messages");
         if (popup != null) gucCreateContextMenu(popup);
    } else { /* firefox */
        var popup = document.getElementById("contentAreaContextMenu");
        if (popup != null)  gucCreateContextMenu(popup);
    }
}
function gucCreateContextMenu(popup)
{
    // destroy old menu items that hold the guc urls
    var menuSep = document.getElementById("context-sep-guc");
    if (menuSep != null) popup.removeChild(menuSep);
    var menu = document.getElementById("context-menu-guc");
    if (menu != null) {
        popup.removeChild(menu);
    } else {
        var i=1;
        var mi;
        while(true) {
            mi = document.getElementById("guc_menuitem" + i);
            if (mi == null) break;
            popup.removeChild(mi);
            i++;
        }
    }
    
    // re-create the new menu-item list
    var menuSep = document.createElement("menuseparator");
    menuSep.setAttribute("id", "context-sep-guc");
    popup.appendChild(menuSep);

    var parent = popup;

    var groupRules = gucPrefs.getBoolPref("groupRules");

    var menu;
    if (groupRules) {
        // create the menupopup element
        menu = document.createElement("menu");
        menu.setAttribute("id", "context-menu-guc");
        parent = document.createElement("menupopup");
        menu.appendChild(parent);
        popup.appendChild(menu);
    }


    var i = 1;
    var m;
    while (true) {
        if (!gucPrefs.prefHasUserValue("menutext"+i)) break;
        text = gucPrefs.getCharPref("menutext" + i);
        if (text == "") break;
        m = document.createElement("menuitem");
        m.setAttribute("id", "guc_menuitem" + i);
        m.setAttribute("oncommand", "onGucClick("+ i +")");
        m.setAttribute("onmouseover", "onGucMouseOver(this, " + i + ")");
        m.setAttribute("label", "null");
        parent.appendChild(m);
        i = i + 1;
    }

    // if there were no menuitems
    if (i==1) {
        if (groupRules) menu.hidden = true;
        menuSep.hidden = true;
    }
}

function replace(text, expression, value) 
{ 
    var exp = new RegExp(expression,'g');
    return text.replace(exp, value) 
}

// invoked when the menu is just about to popup
function onGucPopup()
{
    var i = 1;
    var selectedText = gucGetSelectedText();
    var menu = document.getElementById("context-menu-guc");
    var menuSep = document.getElementById("context-sep-guc");

    if (selectedText != "") {
        var item, menutext;
        var words = selectedText.split(" ");
        if (menu != null) {
            menu.hidden = false;
            if (menuSep != null) menuSep.hidden = false;
            menutext = gucPrefs.getCharPref("contextMenuText");
            menutext = replace(menutext, "##", selectedText);
    
            if (menutext.length > 30)
                menutext = menutext.substr(menutext, 30) + "...";
    
            menu.setAttribute("label", menutext);
        }        
        for (i=1;;i++) {
            item = document.getElementById("guc_menuitem" + i);
            if (item == null) break;

            item.setAttribute("tooltiptext", ""); // this is set in mouseOver
            var se = gucPrefs.getCharPref("searchpattern"+i);
            
            menutext = gucPrefs.getCharPref("menutext" + i);
            if (se == "") { // if no search pattern, match everything
            	menutext = replace(menutext, "##", selectedText);
            } else {
                try {
      	    	    var re = new RegExp(se);
                    var match = re.exec(selectedText);
                    if (match == null) {
                        item.hidden = true;
                        continue; // no matches
                    }
                    menutext = replace(menutext, "##", match[0]);
                    for(j=1; j< match.length; j++)
                        menutext = replace(menutext, "#"+j, match[j] == null ? "" : match[j]);
                }
                catch(ex) {
                    menutext = "<Bad RegExp>"; // useful for debugging
                }
            }

            if (menutext.length > 30)
                menutext = menutext.substr(menutext, 30) + "...";
            
            item.hidden = false;
            item.setAttribute("label", menutext);
        }
    } else { // simply right clicked with no text
        if (menu != null) menu.hidden = true;
        if (menuSep != null) menuSep.hidden = true;
        for (i=1;; i++) {
            item = document.getElementById("guc_menuitem" + i);
            if (item == null) break;
            item.hidden = true;
        }
    }
}

// Will get the selected text, format it and pass it back with joinChar seperating 
// individual words
function gucGetSelectedText()
{
    var node = document.popupNode;
    var selection = "";
    var nodeLocalName = node.localName.toUpperCase();
    if ((nodeLocalName == "TEXTAREA") || (nodeLocalName == "INPUT" && node.type == "text")) {
        selection = node.value.substring(node.selectionStart, node.selectionEnd);
    } 
    else {
        var focusedWindow = document.commandDispatcher.focusedWindow;
        selection = focusedWindow.getSelection().toString();
    }

    return selection; 
}

// Returns the final url after substituting matches
function getUrl(n) 
{
    var url = gucPrefs.getCharPref("url" + n);
    var selectedText = gucGetSelectedText();

    var se = gucPrefs.getCharPref("searchpattern"+n);

    if (se == "") { // if no search pattern, match everything
        url = replace(url, "##", escape(selectedText));
    } else {
        var match;
        try {
            var re = new RegExp(se);
            match = re.exec(selectedText);
            url = replace(url, "##", escape(match[0]));

            for(j=1; j< match.length; j++)
                url = replace(url, "#"+j, match[j] == null ? "" : escape(match[j]));
        }
        catch (ex) {
            url = "<Bad RegExp>";
        }
    }
    return url;
}

function getWindowByType(windowType)
{
    const MEDIATOR_CONTRACTID =
        "@mozilla.org/appshell/window-mediator;1";
    const nsIWindowMediator  = Components.interfaces.nsIWindowMediator;

    var windowManager =
        Components.classes[MEDIATOR_CONTRACTID].getService(nsIWindowMediator);

    return windowManager.getMostRecentWindow(windowType);
}

// Called when one of the url's is clicked
function onGucClick(n)
{
    var url = getUrl(n);

    if (gInThunderbird) {
      var messenger = Components.classes["@mozilla.org/messenger;1"].createInstance();         
      messenger = messenger.QueryInterface(Components.interfaces.nsIMessenger);
      messenger.launchExternalURL(url);
    } else if (gInChatzilla) {
        var window = getWindowByType("navigator:browser");
        if (window != null) {
            // When opened from chatzilla, force focus on new tab
            var newTab = window.getBrowser().addTab(url, null, null);
            window.getBrowser().selectedTab = newTab;
//            window.openNewTabWith(url, null, null, true);
        } else
            openDialog(getBrowserURL(), "_blank", "chrome,all,dialog=no", url);
    } else {
        // The tab open here depends on the browser preferences
        openNewTabWith(url, null, null, true);
    }
}

function onGucMouseOver(mi, n) 
{
    var url = getUrl(n);
    mi.setAttribute("tooltiptext", url);
}
